#include <strlst.h>

extern void httpdthread( DWORD ptr );
void html_hdr( tcp_Socket *s, char *title );
void html_tail( tcp_Socket *s );
int http_dump( tcp_Socket  *s, char *base, char *fname, char *ext );

void html_datestring( tcp_Socket *s, char *format, DWORD plusseconds );

stringlist *cgi_getstrings( tcp_Socket *s );
void cgi_freestrings( stringlist *sl );

typedef struct {
    char *ssi_name;
    void (*ssi_fn)( tcp_Socket *s, stringlist *cookies );
} ssi_type;
int http_shtml( tcp_Socket  *s, char *base, char *fname, char *ext,
    ssi_type *ssi_list, stringlist *cookies);


/*
 * user proc prototype
 *
 * void userproc( tcp_Socket *s, char *cmd, char *name, char *ext,
 *    stringlist *cookies )
 */
